@@ -3,13 +3,15 @@
 import datetime as dt
 import json
 import logging as lg
-import math
 import re
+import socket
 import time
 from collections import OrderedDict
 from hashlib import sha1
 from pathlib import Path
+from urllib.parse import urlparse
 
+import numpy as np
 import requests
 from dateutil import parser as date_parser
 
@@ -19,6 +21,9 @@ from . import utils
 from . import utils_geo
 from ._errors import CacheOnlyModeInterrupt
 
+# capture getaddrinfo function to use original later after mutating it
+_original_getaddrinfo = socket.getaddrinfo
+
 
 def _get_osm_filter(network_type):
     """
@@ -43,7 +48,7 @@ def _get_osm_filter(network_type):
     # are tagged as providing certain services
     filters["drive"] = (
         f'["highway"]["area"!~"yes"]{settings.default_access}'
-        f'["highway"!~"abandoned|bridleway|construction|corridor|cycleway|elevator|'
+        f'["highway"!~"abandoned|bridleway|bus_guideway|construction|corridor|cycleway|elevator|'
         f"escalator|footway|path|pedestrian|planned|platform|proposed|raceway|service|"
         f'steps|track"]'
         f'["motor_vehicle"!~"no"]["motorcar"!~"no"]'
@@ -53,7 +58,7 @@ def _get_osm_filter(network_type):
     # drive+service: allow ways tagged 'service' but filter out certain types
     filters["drive_service"] = (
         f'["highway"]["area"!~"yes"]{settings.default_access}'
-        f'["highway"!~"abandoned|bridleway|construction|corridor|cycleway|elevator|'
+        f'["highway"!~"abandoned|bridleway|bus_guideway|construction|corridor|cycleway|elevator|'
         f'escalator|footway|path|pedestrian|planned|platform|proposed|raceway|steps|track"]'
         f'["motor_vehicle"!~"no"]["motorcar"!~"no"]'
         f'["service"!~"emergency_access|parking|parking_aisle|private"]'
@@ -66,7 +71,7 @@ def _get_osm_filter(network_type):
     # filter ignores such cycleways.
     filters["walk"] = (
         f'["highway"]["area"!~"yes"]{settings.default_access}'
-        f'["highway"!~"abandoned|construction|cycleway|motor|planned|platform|'
+        f'["highway"!~"abandoned|bus_guideway|construction|cycleway|motor|planned|platform|'
         f'proposed|raceway"]'
         f'["foot"!~"no"]["service"!~"private"]'
     )
@@ -75,7 +80,7 @@ def _get_osm_filter(network_type):
     # specifying biking=no
     filters["bike"] = (
         f'["highway"]["area"!~"yes"]{settings.default_access}'
-        f'["highway"!~"abandoned|construction|corridor|elevator|escalator|footway|'
+        f'["highway"!~"abandoned|bus_guideway|construction|corridor|elevator|escalator|footway|'
         f'motor|planned|platform|proposed|raceway|steps"]'
         f'["bicycle"!~"no"]["service"!~"private"]'
     )
@@ -96,7 +101,7 @@ def _get_osm_filter(network_type):
 
     if network_type in filters:
         osm_filter = filters[network_type]
-    else:
+    else:  # pragma: no cover
         raise ValueError(f'Unrecognized network_type "{network_type}"')
 
     return osm_filter
@@ -245,15 +250,57 @@ def _get_http_headers(user_agent=None, referer=None, accept_language=None):
     return headers
 
 
-def _get_pause(recursive_delay=5, default_duration=60):
+def _config_dns(url):
+    """
+    Force socket.getaddrinfo to use IP address instead of host.
+
+    Resolves the URL's domain to an IP address so that we use the same server
+    for both 1) checking the necessary pause duration and 2) sending the query
+    itself even if there is round-robin redirecting among multiple server
+    machines on the server-side. Mutates the getaddrinfo function so it uses
+    the same IP address everytime it finds the host name in the URL.
+
+    For example, the domain overpass-api.de just redirects to one of its
+    subdomains (currently z.overpass-api.de and lz4.overpass-api.de). So if we
+    check the status endpoint of overpass-api.de, we may see results for
+    subdomain z, but when we submit the query itself it gets redirected to
+    subdomain lz4. This could result in violating server lz4's slot management
+    timing.
+
+    Parameters
+    ----------
+    url : string
+        the URL to consistently resolve the IP address of
+
+    Returns
+    -------
+    None
+    """
+    host = urlparse(url).netloc.split(":")[0]
+    ip = socket.gethostbyname(host)
+
+    def _getaddrinfo(*args):
+        if args[0] == host:
+            utils.log(f"Resolved {host} to {ip}")
+            return _original_getaddrinfo(ip, *args[1:])
+        else:
+            return _original_getaddrinfo(*args)
+
+    socket.getaddrinfo = _getaddrinfo
+
+
+def _get_pause(base_endpoint, recursive_delay=5, default_duration=60):
     """
     Get a pause duration from the Overpass API status endpoint.
 
     Check the Overpass API status endpoint to determine how long to wait until
-    next slot is available.
+    the next slot is available. You can disable this via the `ox.config`
+    function's `overpass_rate_limit` argument.
 
     Parameters
     ----------
+    base_endpoint : string
+        base Overpass API endpoint (without "/status" at the end)
     recursive_delay : int
         how long to wait between recursive calls if the server is currently
         running a query
@@ -264,8 +311,12 @@ def _get_pause(recursive_delay=5, default_duration=60):
     -------
     pause : int
     """
+    if not settings.overpass_rate_limit:  # pragma: no cover
+        # if overpass rate limiting is False, then there is zero pause
+        return 0
+
     try:
-        url = settings.overpass_endpoint.rstrip("/") + "/status"
+        url = base_endpoint.rstrip("/") + "/status"
         response = requests.get(url, headers=_get_http_headers())
         status = response.text.split("\n")[3]
         status_first_token = status.split(" ")[0]
@@ -288,14 +339,14 @@ def _get_pause(recursive_delay=5, default_duration=60):
         if status_first_token == "Slot":
             utc_time_str = status.split(" ")[3]
             utc_time = date_parser.parse(utc_time_str).replace(tzinfo=None)
-            pause = math.ceil((utc_time - dt.datetime.utcnow()).total_seconds())
+            pause = int(np.ceil((utc_time - dt.datetime.utcnow()).total_seconds()))
             pause = max(pause, 1)
 
         # if first token is 'Currently', it is currently running a query so
         # check back in recursive_delay seconds
         elif status_first_token == "Currently":
             time.sleep(recursive_delay)
-            pause = _get_pause()
+            pause = _get_pause(base_endpoint)
 
         # any other status is unrecognized: log error, return default duration
         else:
@@ -366,7 +417,7 @@ def _create_overpass_query(polygon_coord_str, tags):
 
     # make sure every value in dict is bool, str, or list of str
     error_msg = "tags must be a dict with values of bool, str, or list of str"
-    if not isinstance(tags, dict):
+    if not isinstance(tags, dict):  # pragma: no cover
         raise TypeError(error_msg)
 
     tags_dict = dict()
@@ -379,11 +430,11 @@ def _create_overpass_query(polygon_coord_str, tags):
             tags_dict[key] = [value]
 
         elif isinstance(value, list):
-            if not all(isinstance(s, str) for s in value):
+            if not all(isinstance(s, str) for s in value):  # pragma: no cover
                 raise TypeError(error_msg)
             tags_dict[key] = value
 
-        else:
+        else:  # pragma: no cover
             raise TypeError(error_msg)
 
     # convert the tags dict into a list of {tag:value} dicts
@@ -460,7 +511,7 @@ def _osm_network_download(polygon, network_type, custom_filter):
         f"Got all network data within polygon from API in {len(polygon_coord_strs)} request(s)"
     )
 
-    if settings.cache_only_mode:
+    if settings.cache_only_mode:  # pragma: no cover
         raise CacheOnlyModeInterrupt("settings.cache_only_mode=True")
 
     return response_jsons
@@ -545,7 +596,7 @@ def _osm_place_download(query, by_osmid=False, limit=1, polygon_geojson=1):
             # each time, for caching purposes
             for key in sorted(query):
                 params[key] = query[key]
-        else:
+        else:  # pragma: no cover
             raise TypeError("query must be a dict or a string")
 
     # request the URL, return the JSON
@@ -573,7 +624,7 @@ def nominatim_request(params, request_type="search", pause=1, error_pause=60):
     -------
     response_json : dict
     """
-    if request_type not in {"search", "reverse", "lookup"}:
+    if request_type not in {"search", "reverse", "lookup"}:  # pragma: no cover
         raise ValueError('Nominatim request_type must be "search", "reverse", or "lookup"')
 
     # prepare Nominatim API URL and see if request already exists in cache
@@ -644,9 +695,14 @@ def overpass_request(data, pause=None, error_pause=60):
     -------
     response_json : dict
     """
+    base_endpoint = settings.overpass_endpoint
+
+    # resolve url to same IP even if there is server round-robin redirecting
+    _config_dns(base_endpoint)
+
     # define the Overpass API URL, then construct a GET-style URL as a string to
     # hash to look up/save to cache
-    url = settings.overpass_endpoint.rstrip("/") + "/interpreter"
+    url = base_endpoint.rstrip("/") + "/interpreter"
     prepared_url = requests.Request("GET", url, params=data).prepare().url
     cached_response_json = _retrieve_from_cache(prepared_url, check_remark=True)
 
@@ -657,7 +713,7 @@ def overpass_request(data, pause=None, error_pause=60):
     else:
         # if this URL is not already in the cache, pause, then request it
         if pause is None:
-            this_pause = _get_pause()
+            this_pause = _get_pause(base_endpoint)
         utils.log(f"Pausing {this_pause} seconds before making HTTP POST request")
         time.sleep(this_pause)
 
@@ -682,7 +738,7 @@ def overpass_request(data, pause=None, error_pause=60):
                 # 429 is 'too many requests' and 504 is 'gateway timeout' from
                 # server overload: handle these by pausing then recursively
                 # re-trying until we get a valid response from the server
-                this_pause = error_pause + _get_pause()
+                this_pause = error_pause + _get_pause(base_endpoint)
                 utils.log(f"{domain} returned {sc}: retry in {this_pause} secs", level=lg.WARNING)
                 time.sleep(this_pause)
                 response_json = overpass_request(data, pause, error_pause)
