@@ -10,6 +10,7 @@ import numpy as np
 import pandas as pd
 
 from . import graph
+from . import projection
 from . import settings
 from . import simplification
 from . import utils
@@ -195,7 +196,7 @@ def plot_graph(
     """
     max_node_size = max(node_size) if hasattr(node_size, "__iter__") else node_size
     max_edge_lw = max(edge_linewidth) if hasattr(edge_linewidth, "__iter__") else edge_linewidth
-    if max_node_size <= 0 and max_edge_lw <= 0:
+    if max_node_size <= 0 and max_edge_lw <= 0:  # pragma: no cover
         raise ValueError("Either node_size or edge_linewidth must be > 0 to plot something.")
 
     # create fig, ax as needed
@@ -318,7 +319,7 @@ def plot_graph_route(
     return fig, ax
 
 
-def plot_graph_routes(G, routes, route_colors="r", **pgr_kwargs):
+def plot_graph_routes(G, routes, route_colors="r", route_linewidths=4, **pgr_kwargs):
     """
     Plot several routes along a graph.
 
@@ -330,6 +331,8 @@ def plot_graph_routes(G, routes, route_colors="r", **pgr_kwargs):
         routes as a list of lists of node IDs
     route_colors : string or list
         if string, 1 color for all routes. if list, the colors for each route.
+    route_linewidths : int or list
+        if int, 1 linewidth for all routes. if list, the linewidth for each route.
     pgr_kwargs
         keyword arguments to pass to plot_graph_route
 
@@ -339,22 +342,27 @@ def plot_graph_routes(G, routes, route_colors="r", **pgr_kwargs):
         matplotlib figure, axis
     """
     # check for valid arguments
-    if not all(isinstance(r, list) for r in routes):
+    if not all(isinstance(r, list) for r in routes):  # pragma: no cover
         raise ValueError("routes must be a list of route lists")
-    if len(routes) < 2:
+    if len(routes) < 2:  # pragma: no cover
         raise ValueError("You must pass more than 1 route")
     if isinstance(route_colors, str):
         route_colors = [route_colors] * len(routes)
-    if len(routes) != len(route_colors):
+    if len(routes) != len(route_colors):  # pragma: no cover
         raise ValueError("route_colors list must have same length as routes")
+    if isinstance(route_linewidths, int):
+        route_linewidths = [route_linewidths] * len(routes)
+    if len(routes) != len(route_linewidths):  # pragma: no cover
+        raise ValueError("route_linewidths list must have same length as routes")
 
     # plot the graph and the first route
-    override = {"route", "route_color", "show", "save", "close"}
+    override = {"route", "route_color", "route_linewidth", "show", "save", "close"}
     kwargs = {k: v for k, v in pgr_kwargs.items() if k not in override}
     fig, ax = plot_graph_route(
         G,
         route=routes[0],
         route_color=route_colors[0],
+        route_linewidth=route_linewidths[0],
         show=False,
         save=False,
         close=False,
@@ -364,11 +372,13 @@ def plot_graph_routes(G, routes, route_colors="r", **pgr_kwargs):
     # plot the subsequent routes on top of existing ax
     override.update({"ax"})
     kwargs = {k: v for k, v in pgr_kwargs.items() if k not in override}
-    for route, route_color in zip(routes[1:], route_colors[1:]):
+    r_rc_rlw = zip(routes[1:], route_colors[1:], route_linewidths[1:])
+    for route, route_color, route_linewidth in r_rc_rlw:
         fig, ax = plot_graph_route(
             G,
             route=route,
             route_color=route_color,
+            route_linewidth=route_linewidth,
             show=False,
             save=False,
             close=False,
@@ -474,10 +484,10 @@ def plot_figure_ground(
             truncate_by_edge=True,
         )
         G = simplification.simplify_graph(G, strict=False)
-    else:
+    else:  # pragma: no cover
         raise ValueError("You must pass an address or lat-lng point or graph.")
 
-    # we need an undirected graph to find every edge incident to a node
+    # we need an undirected graph to find every edge incident on a node
     Gu = utils_graph.get_undirected(G)
 
     # for each edge, get a linewidth according to street type
@@ -783,12 +793,12 @@ def _config_ax(ax, crs, bbox, padding):
     ax.get_yaxis().set_visible(False)
 
     # set aspect ratio
-    if crs == settings.default_crs:
-        # if data are not projected, conform aspect ratio to not stretch plot
-        coslat = np.cos((south + north) / 2 / 180 * np.pi)
-        ax.set_aspect(1 / coslat)
-    else:
-        # if projected, make everything square
+    if projection.is_projected(crs):
+        # if projected, make equal aspect ratio
         ax.set_aspect("equal")
+    else:
+        # if not projected, conform aspect ratio to not stretch plot
+        cos_lat = np.cos((south + north) / 2 / 180 * np.pi)
+        ax.set_aspect(1 / cos_lat)
 
     return ax
