@@ -1,8 +1,7 @@
 """Project spatial geometries and spatial networks."""
 
-import math
-
 import geopandas as gpd
+import numpy as np
 from pyproj import CRS
 
 from . import settings
@@ -10,6 +9,25 @@ from . import utils
 from . import utils_graph
 
 
+def is_projected(crs):
+    """
+    Determine if a coordinate reference system is projected or not.
+
+    This is a convenience wrapper around the pyproj.CRS.is_projected function.
+
+    Parameters
+    ----------
+    crs : string or pyproj.CRS
+        the coordinate reference system
+
+    Returns
+    -------
+    projected : bool
+        True if crs is projected, otherwise False
+    """
+    return CRS.from_user_input(crs).is_projected
+
+
 def project_geometry(geometry, crs=None, to_crs=None, to_latlong=False):
     """
     Project a shapely geometry from its current CRS to another.
@@ -69,7 +87,7 @@ def project_gdf(gdf, to_crs=None, to_latlong=False):
     gdf_proj : geopandas.GeoDataFrame
         the projected GeoDataFrame
     """
-    if gdf.crs is None or len(gdf) < 1:
+    if gdf.crs is None or len(gdf) < 1:  # pragma: no cover
         raise ValueError("GeoDataFrame must have a valid CRS and cannot be empty")
 
     # if to_latlong is True, project the gdf to latlong
@@ -84,14 +102,14 @@ def project_gdf(gdf, to_crs=None, to_latlong=False):
 
     # otherwise, automatically project the gdf to UTM
     else:
-        if CRS.from_user_input(gdf.crs).is_projected:
+        if is_projected(gdf.crs):  # pragma: no cover
             raise ValueError("Geometry must be unprojected to calculate UTM zone")
 
         # calculate longitude of centroid of union of all geometries in gdf
         avg_lng = gdf["geometry"].unary_union.centroid.x
 
         # calculate UTM zone from avg longitude to define CRS to project to
-        utm_zone = math.floor((avg_lng + 180) / 6) + 1
+        utm_zone = int(np.floor((avg_lng + 180) / 6) + 1)
         utm_crs = f"+proj=utm +zone={utm_zone} +ellps=WGS84 +datum=WGS84 +units=m +no_defs"
 
         # project the GeoDataFrame to the UTM CRS
