@@ -4,12 +4,11 @@ import logging as lg
 
 import geopandas as gpd
 import networkx as nx
-import pandas as pd
 from shapely.geometry import LineString
 from shapely.geometry import Point
 from shapely.geometry import Polygon
-from shapely.ops import unary_union
 
+from . import stats
 from . import utils
 from . import utils_graph
 
@@ -222,7 +221,7 @@ def simplify_graph(G, strict=True, remove_rings=True):
         topologically simplified graph, with a new `geometry` attribute on
         each simplified edge
     """
-    if "simplified" in G.graph and G.graph["simplified"]:
+    if "simplified" in G.graph and G.graph["simplified"]:  # pragma: no cover
         raise Exception("This graph has already been simplified, cannot simplify it again.")
 
     utils.log("Begin topologically simplifying the graph...")
@@ -374,9 +373,7 @@ def consolidate_intersections(
     """
     # if dead_ends is False, discard dead-ends to retain only intersections
     if not dead_ends:
-        spn = nx.get_node_attributes(G, "street_count")
-        if set(spn) != set(G.nodes):
-            utils.log("Graph nodes changed since `street_count`s were calculated", level=lg.WARN)
+        spn = stats.streets_per_node(G)
         dead_end_nodes = [node for node, count in spn.items() if count <= 1]
 
         # make a copy to not mutate original graph object caller passed in
@@ -399,64 +396,31 @@ def consolidate_intersections(
             return _merge_nodes_geometric(G, tolerance).centroid
 
 
-def _merge_nodes_geometric(G, tolerance, chunk=True):
+def _merge_nodes_geometric(G, tolerance):
     """
     Geometrically merge nodes within some distance of each other.
 
-    If chunk=True, it sorts the nodes GeoSeries by geometry x and y values (to
-    make unary_union faster), then buffers by tolerance. Next it divides the
-    nodes GeoSeries into n-sized chunks, where n = the square root of the
-    number of nodes. Then it runs unary_union on each chunk, and then runs
-    unary_union on the resulting unary unions. This is much faster on large
-    graphs (n>100000) because of how unary_union's runtime scales with vertex
-    count. But chunk=False is usually faster on small and medium sized graphs.
-    This hacky method will hopefully be made obsolete when shapely becomes
-    vectorized by incorporating the pygeos codebase.
-
     Parameters
     ----------
     G : networkx.MultiDiGraph
         a projected graph
     tolerance : float
-        nodes are buffered to this distance (in graph's geometry's units) and
-        subsequent overlaps are dissolved into a single polygon
-    chunk : bool
-        if True, divide nodes into geometrically sorted chunks to improve the
-        speed of unary_union operation by running it on each chunk and then
-        running it on the results of those runs
+        buffer nodes to this distance (in graph's geometry's units) then merge
+        overlapping polygons into a single polygon via a unary union operation
 
     Returns
     -------
-    merged_nodes : GeoSeries
+    merged : GeoSeries
         the merged overlapping polygons of the buffered nodes
     """
-    # yield n-sized chunks of list one at a time
-    def get_chunks(lst, n):
-        for i in range(0, len(lst), n):
-            yield lst[i : i + n]
-
-    gs_nodes = utils_graph.graph_to_gdfs(G, edges=False)["geometry"]
-
-    if not chunk:
-        # buffer nodes GeoSeries then get unary union to merge overlaps
-        merged_nodes = gs_nodes.buffer(tolerance).unary_union
-
-    else:
-        # sort nodes GeoSeries by geometry x/y vals to make unary_union faster
-        idx = pd.DataFrame((gs_nodes.x, gs_nodes.y)).T.sort_values([0, 1]).index
-
-        # buffer nodes by tolerance then generate n-sized chunks of nodes
-        n = int(len(G) ** 0.5)
-        chunks = get_chunks(gs_nodes.loc[idx].buffer(tolerance).values, n)
-
-        # unary_union each chunk then unary_union those unary unions
-        merged_nodes = unary_union([unary_union(chunk) for chunk in chunks])
+    # buffer nodes GeoSeries then get unary union to merge overlaps
+    merged = utils_graph.graph_to_gdfs(G, edges=False)["geometry"].buffer(tolerance).unary_union
 
     # if only a single node results, make it iterable to convert to GeoSeries
-    if isinstance(merged_nodes, Polygon):
-        merged_nodes = [merged_nodes]
+    if isinstance(merged, Polygon):
+        merged = [merged]
 
-    return gpd.GeoSeries(list(merged_nodes), crs=G.graph["crs"])
+    return gpd.GeoSeries(list(merged), crs=G.graph["crs"])
 
 
 def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=True):
@@ -533,6 +497,9 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
                     gdf.loc[wcc, "cluster"] = f"{cluster_label}-{suffix}"
                     suffix += 1
 
+    # give nodes unique integer IDs (subclusters with suffixes are strings)
+    gdf["cluster"] = gdf["cluster"].factorize()[0]
+
     # STEP 4
     # create new empty graph and copy over misc graph data
     H = nx.MultiDiGraph()
@@ -595,7 +562,7 @@ def _consolidate_intersections_rebuild_graph(G, tolerance=10, reconnect_edges=Tr
             y = H.nodes[cluster_label]["y"]
             xy = [(x, y)]
 
-            # for each edge incident to this new merged node, update its
+            # for each edge incident on this new merged node, update its
             # geometry to extend to/from the new node's point coords
             in_edges = set(H.in_edges(cluster_label, keys=True))
             out_edges = set(H.out_edges(cluster_label, keys=True))
