@@ -16,7 +16,7 @@ def citation():
 
     Boeing, G. 2017. OSMnx: New Methods for Acquiring, Constructing, Analyzing,
     and Visualizing Complex Street Networks. Computers, Environment and Urban
-    Systems, 65(126-139). https://doi.org/10.1016/j.compenvurbsys.2017.05.004
+    Systems, 65, 126-139. https://doi.org/10.1016/j.compenvurbsys.2017.05.004
 
     Returns
     -------
@@ -26,7 +26,7 @@ def citation():
         "Citation:\n\n"
         "Boeing, G. 2017. OSMnx: New Methods for Acquiring, "
         "Constructing, Analyzing, and Visualizing Complex Street "
-        "Networks. Computers, Environment and Urban Systems, 65(126-139). "
+        "Networks. Computers, Environment and Urban Systems, 65, 126-139. "
         "https://doi.org/10.1016/j.compenvurbsys.2017.05.004\n\n"
         "BibTeX entry for LaTeX users:\n\n"
         "@article{boeing_osmnx_2017,\n"
@@ -69,7 +69,7 @@ def ts(style="datetime", template=None):
             template = "{:%Y-%m-%d}"
         elif style == "time":
             template = "{:%H:%M:%S}"
-        else:
+        else:  # pragma: no cover
             raise ValueError(f'unrecognized timestamp style "{style}"')
 
     ts = template.format(dt.datetime.now())
@@ -104,6 +104,7 @@ def config(
     osm_xml_way_attrs=settings.osm_xml_way_attrs,
     osm_xml_way_tags=settings.osm_xml_way_tags,
     overpass_endpoint=settings.overpass_endpoint,
+    overpass_rate_limit=settings.overpass_rate_limit,
     overpass_settings=settings.overpass_settings,
     timeout=settings.timeout,
     use_cache=settings.use_cache,
@@ -122,6 +123,8 @@ def config(
         Only use if specifically saving to .osm XML file with save_graph_xml
         function. if True, forces all ways to be loaded as oneway ways,
         preserving the original order of nodes stored in the OSM way XML.
+        This also retains original OSM string values for oneway attribute
+        values, rather than converting them to a True/False bool.
     bidirectional_network_types : list
         network types for which a fully bidirectional graph will be created
     cache_folder : string or pathlib.Path
@@ -169,7 +172,7 @@ def config(
         Overpass server memory allocation size for the query, in bytes. If
         None, server will use its default allocation size. Use with caution.
     nominatim_endpoint : string
-        the API endpoint to use for nominatim queries
+        base API endpoint to use for nominatim queries
     nominatim_key : string
         your API key, if you are using an endpoint that requires one
     osm_xml_node_attrs : list
@@ -181,11 +184,15 @@ def config(
     osm_xml_way_tags : list
         edge tags for for saving .osm XML files with save_graph_xml function
     overpass_endpoint : string
-        the API endpoint to use for overpass queries
+        base API endpoint to use for overpass queries
+    overpass_rate_limit : bool
+        if True, check the overpass server status endpoint for how long to
+        pause before making request. Necessary if server uses slot management,
+        but can be set to False if you are running your own overpass instance.
     overpass_settings : string
         Settings string for overpass queries. For example, to query historical
         OSM data as of a certain date:
-        `'[out:json][timeout:90][date:"2019-10-28T19:20:00Z"]'`.
+        ``'[out:json][timeout:90][date:"2019-10-28T19:20:00Z"]'``.
         Use with caution.
     timeout : int
         the timeout interval for the HTTP request and for API to use while
@@ -230,6 +237,7 @@ def config(
     settings.osm_xml_way_attrs = osm_xml_way_attrs
     settings.osm_xml_way_tags = osm_xml_way_tags
     settings.overpass_endpoint = overpass_endpoint
+    settings.overpass_rate_limit = overpass_rate_limit
     settings.overpass_settings = overpass_settings
     settings.timeout = timeout
     settings.use_cache = use_cache
@@ -302,7 +310,7 @@ def log(message, level=None, name=None, filename=None):
         sys.stdout = standard_out
 
 
-def _get_logger(level=None, name=None, filename=None):
+def _get_logger(level, name, filename):
     """
     Create a logger or return the current one if already instantiated.
 
@@ -319,13 +327,6 @@ def _get_logger(level=None, name=None, filename=None):
     -------
     logger : logging.logger
     """
-    if level is None:
-        level = settings.log_level
-    if name is None:
-        name = settings.log_name
-    if filename is None:
-        filename = settings.log_filename
-
     logger = lg.getLogger(name)
 
     # if a logger with this name is not already set up
