@@ -14,6 +14,7 @@ from collections import OrderedDict
 from pathlib import Path
 
 import folium
+import geopandas as gpd
 import networkx as nx
 import numpy as np
 import pandas as pd
@@ -53,6 +54,7 @@ polygon = wkt.loads(p)
 
 def test_logging():
     # test OSMnx's logger
+    ox.log("test a fake default message")
     ox.log("test a fake debug", level=lg.DEBUG)
     ox.log("test a fake info", level=lg.INFO)
     ox.log("test a fake warning", level=lg.WARNING)
@@ -63,7 +65,7 @@ def test_logging():
     ox.ts(style="time")
 
 
-def test_geometry_coords_rounding():
+def test_coords_rounding():
     # test the rounding of geometry coordinates
     precision = 3
 
@@ -82,6 +84,9 @@ def test_geometry_coords_rounding():
             [(11.123456, 12.123456), (13.123456, 14.123456)],
         ]
     )
+
+    _ = ox.utils_geo.redistribute_vertices(shape1, 0.01)
+
     shape2 = ox.utils_geo.round_geometry_coords(shape1, precision)
 
     shape1 = Polygon([(1.123456, 2.123456), (3.123456, 4.123456), (6.123456, 5.123456)])
@@ -98,11 +103,50 @@ def test_geometry_coords_rounding():
 
 def test_geocode_to_gdf():
     # test loading spatial boundaries and plotting
+    city = ox.geocode_to_gdf("R2999176", by_osmid=True)
     city = ox.geocode_to_gdf(place1, which_result=1, buffer_dist=100)
     city_projected = ox.project_gdf(city, to_crs="epsg:3395")
 
 
-def test_graph_from_xml():
+def test_stats():
+    # create graph, add a new node, add bearings, project it
+    G = ox.graph_from_point(location_point, dist=500, network_type="drive")
+    G.add_node(0, x=location_point[1], y=location_point[0])
+    _ = ox.bearing.get_bearing((0, 0), (1, 1))
+    G = ox.add_edge_bearings(G)
+    G_proj = ox.project_graph(G)
+
+    # calculate stats
+    cspn = ox.utils_graph.count_streets_per_node(G)
+    stats = ox.basic_stats(G)
+    stats = ox.basic_stats(G, area=1000)
+    stats = ox.basic_stats(
+        G_proj, area=1000, clean_intersects=True, tolerance=15, circuity_dist="euclidean"
+    )
+
+    # calculate extended stats
+    stats = ox.extended_stats(G, connectivity=True, anc=False, ecc=True, bc=True, cc=True)
+
+    # calculate entropy
+    Gu = ox.get_undirected(G)
+    entropy = ox.bearing.orientation_entropy(Gu, weight="length")
+    fig, ax = ox.bearing.plot_orientation(Gu, area=True, title="Title")
+    fig, ax = ox.bearing.plot_orientation(Gu, ax=ax, area=False, title="Title")
+
+    # test cleaning and rebuilding graph
+    G_clean = ox.consolidate_intersections(G_proj, tolerance=10, rebuild_graph=True, dead_ends=True)
+    G_clean = ox.consolidate_intersections(
+        G_proj, tolerance=10, rebuild_graph=True, reconnect_edges=False
+    )
+    G_clean = ox.consolidate_intersections(G_proj, tolerance=10, rebuild_graph=False)
+
+    # try consolidating an empty graph
+    G = nx.MultiDiGraph(crs="epsg:4326")
+    G_clean = ox.consolidate_intersections(G, rebuild_graph=True)
+    G_clean = ox.consolidate_intersections(G, rebuild_graph=False)
+
+
+def test_osm_xml():
     # test loading a graph from a local .osm xml file
     node_id = 53098262
     neighbor_ids = 53092170, 53060438, 53027353, 667744075
@@ -124,34 +168,76 @@ def test_graph_from_xml():
 
     os.remove(temp_filename)
 
+    # test .osm xml saving
+    default_all_oneway = ox.settings.all_oneway
+    ox.settings.all_oneway = True
+    G = ox.graph_from_point(location_point, dist=500, network_type="drive")
+    ox.io.save_graph_xml(G, merge_edges=False)
+
+    # test osm xml output merge edges
+    ox.io.save_graph_xml(G, merge_edges=True, edge_tag_aggs=[("length", "sum")])
+
+    # test osm xml output from gdfs
+    nodes, edges = ox.graph_to_gdfs(G)
+    ox.io.save_graph_xml([nodes, edges])
+
+    # test ordered nodes from way
+    df = pd.DataFrame({"u": [54, 2, 5, 3, 10, 19, 20], "v": [76, 3, 8, 10, 5, 20, 15]})
+    ordered_nodes = ox.osm_xml._get_unique_nodes_ordered_from_way(df)
+    assert ordered_nodes == [2, 3, 10, 5, 8]
+
+    ox.settings.all_oneway = default_all_oneway
+
+
+def test_elevation():
+
+    G = ox.graph_from_address(address=address, dist=500, dist_type="bbox", network_type="bike")
+    rasters = list(Path("tests/input_data").glob("elevation*.tif"))
+
+    # add node elevations from a single raster file (some nodes will be null)
+    G = ox.elevation.add_node_elevations_raster(G, rasters[0], cpus=1)
+
+    # add node elevations from multiple raster files
+    G = ox.elevation.add_node_elevations_raster(G, rasters)
+    assert pd.notnull(pd.Series(dict(G.nodes(data="elevation")))).all()
+
+    # add edge grades and their absolute values
+    G = ox.add_edge_grades(G, add_absolute=True)
+
 
 def test_routing():
 
     G = ox.graph_from_address(address=address, dist=500, dist_type="bbox", network_type="bike")
 
-    # give each node a random elevation then calculate edge grades
-    randm = np.random.random(size=len(G))
-    elevs = {n: e for n, e in zip(G.nodes(), randm)}
-    nx.set_node_attributes(G, name="elevation", values=elevs)
-    G = ox.add_edge_grades(G, add_absolute=True)
-
     # give each edge speed and travel time attributes
     G = ox.add_edge_speeds(G)
+    G = ox.add_edge_speeds(G, hwy_speeds={"motorway": 100})
     G = ox.add_edge_travel_times(G)
 
-    orig_node = list(G.nodes())[5]
-    dest_node = list(G.nodes())[-5]
-    orig_pt = (G.nodes[orig_node]["y"], G.nodes[orig_node]["x"])
-    dest_pt = (G.nodes[dest_node]["y"], G.nodes[dest_node]["x"])
-    route = ox.shortest_path(G, orig_node, dest_node, weight="travel_time")
+    orig_x = np.array([-122.404771])
+    dest_x = np.array([-122.401429])
+    orig_y = np.array([37.794302])
+    dest_y = np.array([37.794987])
+    orig_node = ox.distance.nearest_nodes(G, orig_x, orig_y)[0]
+    dest_node = ox.distance.nearest_nodes(G, dest_x, dest_y)[0]
 
+    route = ox.shortest_path(G, orig_node, dest_node, weight="travel_time")
+    attributes = ox.utils_graph.get_route_edge_attributes(G, route)
     attributes = ox.utils_graph.get_route_edge_attributes(G, route, "travel_time")
 
     fig, ax = ox.plot_graph_route(G, route, save=True)
 
-    fig, ax = ox.plot_graph_route(G, route, save=True)
+    # test multiple origins-destinations
+    n = 5
+    nodes = np.array(G.nodes)
+    origs = np.random.choice(nodes, size=n, replace=True)
+    dests = np.random.choice(nodes, size=n, replace=True)
+    paths1 = ox.shortest_path(G, origs, dests, weight="length", cpus=1)
+    paths2 = ox.shortest_path(G, origs, dests, weight="length", cpus=2)
+    paths3 = ox.shortest_path(G, origs, dests, weight="length", cpus=None)
+    assert paths1 == paths2 == paths3
 
-    # test multiple routes
+    # test k shortest paths
     routes = ox.k_shortest_paths(G, orig_node, dest_node, k=2, weight="travel_time")
     fig, ax = ox.plot_graph_routes(G, list(routes))
 
@@ -170,6 +256,7 @@ def test_routing():
 
 def test_plots():
     G = ox.graph_from_point(location_point, dist=500, network_type="drive")
+    Gp = ox.project_graph(G)
 
     # test getting colors
     co = ox.plot.get_colors(n=5, return_hex=True)
@@ -179,6 +266,8 @@ def test_plots():
     # plot and save to disk
     filepath = Path(ox.settings.data_folder) / "test.svg"
     fig, ax = ox.plot_graph(G, show=False, save=True, close=True, filepath=filepath)
+    fig, ax = ox.plot_graph(Gp, edge_linewidth=0)
+
     fig, ax = ox.plot_graph(
         G,
         figsize=(5, 5),
@@ -205,35 +294,36 @@ def test_plots():
 
 def test_find_nearest():
 
-    # get graph
+    # get graph and x/y coords to search
     G = ox.graph_from_point(location_point, dist=500, network_type="drive")
-
-    # convert graph to node/edge GeoDataFrames and back again
-    gdf_nodes, gdf_edges = ox.graph_to_gdfs(
-        G, nodes=True, edges=True, node_geometry=True, fill_edge_geometry=True
-    )
-    assert len(gdf_nodes) == len(G)
-    assert len(gdf_edges) == len(G.edges(keys=True))
-    G = ox.graph_from_gdfs(gdf_nodes, gdf_edges)
-    assert len(gdf_nodes) == len(G)
-    assert len(gdf_edges) == len(G.edges(keys=True))
+    Gp = ox.project_graph(G)
+    points = ox.utils_geo.sample_points(ox.get_undirected(Gp), 5)
+    X = points.x.values
+    Y = points.y.values
 
     # get nearest node
-    nn, d = ox.get_nearest_node(G, location_point, method="euclidean", return_dist=True)
+    nn, d = ox.get_nearest_node(Gp, location_point, method="euclidean", return_dist=True)
+    nn = ox.get_nearest_node(Gp, location_point, method="euclidean", return_dist=False)
 
-    # get nearest nodes: haversine, kdtree, balltree
-    X = gdf_nodes["x"].head()
-    Y = gdf_nodes["y"].head()
-    nn1 = ox.get_nearest_nodes(G, X, Y)
-    nn2 = ox.get_nearest_nodes(G, X, Y, method="kdtree")
-    nn3 = ox.get_nearest_nodes(G, X, Y, method="balltree")
+    # get nearest nodes
+    nn0, dist0 = ox.distance.nearest_nodes(G, X[0], Y[0], return_dist=True)
+    nn1, dist1 = ox.get_nearest_nodes(G, X, Y, return_dist=True)
+    nn2, dist2 = ox.get_nearest_nodes(Gp, X, Y, method="kdtree", return_dist=True)
+    nn3, dist3 = ox.get_nearest_nodes(G, X, Y, method="balltree", return_dist=True)
+    nn4 = ox.get_nearest_nodes(G, X, Y)
+    nn5 = ox.get_nearest_nodes(Gp, X, Y, method="kdtree")
+    nn6 = ox.get_nearest_nodes(G, X, Y, method="balltree")
 
     # get nearest edge
-    u, v, k, g, d = ox.get_nearest_edge(G, location_point, return_geom=True, return_dist=True)
+    u, v, k, g, d = ox.get_nearest_edge(Gp, location_point, return_geom=True, return_dist=True)
+    u, v, k, g = ox.get_nearest_edge(Gp, location_point, return_geom=True)
+    u, v, k, d = ox.get_nearest_edge(Gp, location_point, return_dist=True)
+    u, v, k = ox.get_nearest_edge(Gp, location_point)
 
-    # get nearest edges: haversine, kdtree, balltree
-    ne1 = ox.get_nearest_edges(G, X, Y)
-    ne2 = ox.get_nearest_edges(G, X, Y, method="kdtree")
+    # get nearest edges
+    ne0 = ox.distance.nearest_edges(Gp, X[0], Y[0], interpolate=50)
+    ne1 = ox.get_nearest_edges(Gp, X, Y)
+    ne2 = ox.get_nearest_edges(Gp, X, Y, method="kdtree")
     ne3 = ox.get_nearest_edges(G, X, Y, method="balltree", dist=0.0001)
 
 
@@ -288,62 +378,68 @@ def test_api_endpoints():
     ox.settings.overpass_endpoint = default_overpass_endpoint
 
 
-def test_network_saving_loading():
+def test_graph_save_load():
 
     # save graph as shapefile and geopackage
     G = ox.graph_from_place(place1, network_type="drive")
     ox.save_graph_shapefile(G)
-    ox.save_graph_geopackage(G)
+    ox.save_graph_geopackage(G, directed=False)
+
+    # save/load geopackage and convert graph to/from node/edge GeoDataFrames
+    fp = ".temp/data/graph-dir.gpkg"
+    ox.save_graph_geopackage(G, filepath=fp, directed=True)
+    gdf_nodes1 = gpd.read_file(fp, layer="nodes").set_index("osmid")
+    gdf_edges1 = gpd.read_file(fp, layer="edges").set_index(["u", "v", "key"])
+    G2 = ox.graph_from_gdfs(gdf_nodes1, gdf_edges1)
+    G2 = ox.graph_from_gdfs(gdf_nodes1, gdf_edges1, graph_attrs=G.graph)
+    gdf_nodes2, gdf_edges2 = ox.graph_to_gdfs(G2)
+    assert set(gdf_nodes1.index) == set(gdf_nodes2.index) == set(G.nodes) == set(G2.nodes)
+    assert set(gdf_edges1.index) == set(gdf_edges2.index) == set(G.edges) == set(G2.edges)
+
+    # create random boolean graph/node/edge attributes
+    attr_name = "test_bool"
+    G.graph[attr_name] = False
+    bools = np.random.randint(0, 2, len(G.nodes))
+    node_attrs = {n: bool(b) for n, b in zip(G.nodes, bools)}
+    nx.set_node_attributes(G, node_attrs, attr_name)
+    bools = np.random.randint(0, 2, len(G.edges))
+    edge_attrs = {n: bool(b) for n, b in zip(G.edges, bools)}
+    nx.set_edge_attributes(G, edge_attrs, attr_name)
 
     # save/load graph as graphml file
     ox.save_graphml(G, gephi=True)
     ox.save_graphml(G, gephi=False)
     filepath = Path(ox.settings.data_folder) / "graph.graphml"
-    G2 = ox.load_graphml(filepath)
+    G2 = ox.load_graphml(
+        filepath,
+        graph_dtypes={attr_name: ox.io._convert_bool_string},
+        node_dtypes={attr_name: ox.io._convert_bool_string},
+        edge_dtypes={attr_name: ox.io._convert_bool_string},
+    )
 
     # verify everything in G is equivalent in G2
-    for (n1, d1), (n2, d2) in zip(G.nodes(data=True), G2.nodes(data=True)):
+    assert tuple(G.graph.keys()) == tuple(G2.graph.keys())
+    assert tuple(G.graph.values()) == tuple(G2.graph.values())
+    z = zip(G.nodes(data=True), G2.nodes(data=True))
+    for (n1, d1), (n2, d2) in z:
         assert n1 == n2
-        assert d1 == d2
-    for (u1, v1, k1, d1), (u2, v2, k2, d2) in zip(
-        G.edges(keys=True, data=True), G2.edges(keys=True, data=True)
-    ):
+        assert tuple(d1.keys()) == tuple(d2.keys())
+        assert tuple(d1.values()) == tuple(d2.values())
+    z = zip(G.edges(keys=True, data=True), G2.edges(keys=True, data=True))
+    for (u1, v1, k1, d1), (u2, v2, k2, d2) in z:
         assert u1 == u2
         assert v1 == v2
         assert k1 == k2
         assert tuple(d1.keys()) == tuple(d2.keys())
         assert tuple(d1.values()) == tuple(d2.values())
-    for (k1, v1), (k2, v2) in zip(G.graph.items(), G2.graph.items()):
-        assert k1 == k2
-        assert v1 == v2
 
     # test custom data types
     nd = {"osmid": str}
     ed = {"length": str, "osmid": float}
     G2 = ox.load_graphml(filepath, node_dtypes=nd, edge_dtypes=ed)
 
-    # test osm xml output
-    default_all_oneway = ox.settings.all_oneway
-    ox.settings.all_oneway = True
-    G = ox.graph_from_point(location_point, dist=500, network_type="drive")
-    ox.save_graph_xml(G, merge_edges=False)
 
-    # test osm xml output merge edges
-    ox.save_graph_xml(G, merge_edges=True, edge_tag_aggs=[("length", "sum")])
-
-    # test osm xml output from gdfs
-    nodes, edges = ox.graph_to_gdfs(G)
-    ox.save_graph_xml([nodes, edges])
-
-    # test ordered nodes from way
-    df = pd.DataFrame({"u": [54, 2, 5, 3, 10, 19, 20], "v": [76, 3, 8, 10, 5, 20, 15]})
-    ordered_nodes = ox.osm_xml._get_unique_nodes_ordered_from_way(df)
-    assert ordered_nodes == [2, 3, 10, 5, 8]
-
-    ox.settings.all_oneway = default_all_oneway
-
-
-def test_get_network_methods():
+def test_graph_from_functions():
 
     # graph from bounding box
     _ = ox.utils_geo.bbox_from_point(location_point, project_utm=True, return_crs=True)
@@ -387,26 +483,6 @@ def test_get_network_methods():
     )
 
 
-def test_stats():
-    # create graph, add bearings, project it
-    G = ox.graph_from_point(location_point, dist=500, network_type="drive")
-    G = ox.add_edge_bearings(G)
-    G_proj = ox.project_graph(G)
-
-    # calculate stats
-    stats = ox.basic_stats(G)
-    stats = ox.basic_stats(G, area=1000)
-    stats = ox.basic_stats(
-        G_proj, area=1000, clean_intersects=True, tolerance=15, circuity_dist="euclidean"
-    )
-
-    # calculate extended stats
-    stats = ox.extended_stats(G, connectivity=True, anc=False, ecc=True, bc=True, cc=True)
-
-    # test cleaning and rebuilding graph
-    G_clean = ox.consolidate_intersections(G_proj, tolerance=10, rebuild_graph=True, dead_ends=True)
-
-
 def test_geometries():
 
     # geometries_from_bbox - bounding box query to return empty GeoDataFrame
@@ -423,6 +499,7 @@ def test_geometries():
 
     # geometries_from_place - includes test of list of places
     tags = {"amenity": True, "landuse": ["retail", "commercial"], "highway": "bus_stop"}
+    gdf = ox.geometries_from_place(place1, tags=tags)
     gdf = ox.geometries_from_place([place1], tags=tags)
 
     # geometries_from_address - includes testing overpass settings and snapshot from 2019
